@@ -16,6 +16,9 @@ MAX_NUM_FORM_COUNT = 'MAX_NUM_FORMS'
 ORDERING_FIELD_NAME = 'ORDER'
 DELETION_FIELD_NAME = 'DELETE'
 
+# default maximum number of forms in a formset, to prevent memory exhaustion
+DEFAULT_MAX_NUM = 1000
+
 class ManagementForm(Form):
     """
     ``ManagementForm`` is used to keep track of how many form instances
@@ -37,8 +40,8 @@ class BaseFormSet(StrAndUnicode):
         self.is_bound = data is not None or files is not None
         self.prefix = prefix or self.get_default_prefix()
         self.auto_id = auto_id
-        self.data = data
-        self.files = files
+        self.data = data or {}
+        self.files = files or {}
         self.initial = initial
         self.error_class = error_class
         self._errors = None
@@ -49,9 +52,20 @@ class BaseFormSet(StrAndUnicode):
     def __unicode__(self):
         return self.as_table()
 
+    def __iter__(self):
+        """Yields the forms in the order they should be rendered"""
+        return iter(self.forms)
+
+    def __getitem__(self, index):
+        """Returns the form at the given index, based on the rendering order"""
+        return list(self)[index]
+
+    def __len__(self):
+        return len(self.forms)
+
     def _management_form(self):
         """Returns the ManagementForm instance for this FormSet."""
-        if self.data or self.files:
+        if self.is_bound:
             form = ManagementForm(self.data, auto_id=self.auto_id, prefix=self.prefix)
             if not form.is_valid():
                 raise ValidationError('ManagementForm data is missing or has been tampered with')
@@ -66,7 +80,7 @@ class BaseFormSet(StrAndUnicode):
 
     def total_form_count(self):
         """Returns the total number of forms in this FormSet."""
-        if self.data or self.files:
+        if self.is_bound:
             return self.management_form.cleaned_data[TOTAL_FORM_COUNT]
         else:
             initial_forms = self.initial_form_count()
@@ -81,7 +95,7 @@ class BaseFormSet(StrAndUnicode):
 
     def initial_form_count(self):
         """Returns the number of forms that are required in this FormSet."""
-        if self.data or self.files:
+        if self.is_bound:
             return self.management_form.cleaned_data[INITIAL_FORM_COUNT]
         else:
             # Use the length of the inital data if it's there, 0 otherwise.
@@ -93,7 +107,7 @@ class BaseFormSet(StrAndUnicode):
     def _construct_forms(self):
         # instantiate all the forms and put them in self.forms
         self.forms = []
-        for i in xrange(self.total_form_count()):
+        for i in xrange(min(self.total_form_count(), self.absolute_max)):
             self.forms.append(self._construct_form(i))
 
     def _construct_form(self, i, **kwargs):
@@ -101,7 +115,7 @@ class BaseFormSet(StrAndUnicode):
         Instantiates and returns the i-th form instance in a formset.
         """
         defaults = {'auto_id': self.auto_id, 'prefix': self.add_prefix(i)}
-        if self.data or self.files:
+        if self.is_bound:
             defaults['data'] = self.data
             defaults['files'] = self.files
         if self.initial:
@@ -196,14 +210,12 @@ class BaseFormSet(StrAndUnicode):
             # A sort function to order things numerically ascending, but
             # None should be sorted below anything else. Allowing None as
             # a comparison value makes it so we can leave ordering fields
-            # blamk.
-            def compare_ordering_values(x, y):
-                if x[1] is None:
-                    return 1
-                if y[1] is None:
-                    return -1
-                return x[1] - y[1]
-            self._ordering.sort(compare_ordering_values)
+            # blank.
+            def compare_ordering_key(k):
+                if k[1] is None:
+                    return (1, 0) # +infinity, larger than any number
+                return (0, k[1])
+            self._ordering.sort(key=compare_ordering_key)
         # Return a list of form.cleaned_data dicts in the order spcified by
         # the form data.
         return [self.forms[i[0]] for i in self._ordering]
@@ -323,15 +335,30 @@ class BaseFormSet(StrAndUnicode):
         # XXX: there is no semantic division between forms here, there
         # probably should be. It might make sense to render each form as a
         # table row with each field as a td.
-        forms = u' '.join([form.as_table() for form in self.forms])
+        forms = u' '.join([form.as_table() for form in self])
+        return mark_safe(u'\n'.join([unicode(self.management_form), forms]))
+
+    def as_p(self):
+        "Returns this formset rendered as HTML <p>s."
+        forms = u' '.join([form.as_p() for form in self])
+        return mark_safe(u'\n'.join([unicode(self.management_form), forms]))
+
+    def as_ul(self):
+        "Returns this formset rendered as HTML <li>s."
+        forms = u' '.join([form.as_ul() for form in self])
         return mark_safe(u'\n'.join([unicode(self.management_form), forms]))
 
 def formset_factory(form, formset=BaseFormSet, extra=1, can_order=False,
                     can_delete=False, max_num=None):
     """Return a FormSet for the given form class."""
+    if max_num is None:
+        max_num = DEFAULT_MAX_NUM
+    # hard limit on forms instantiated, to prevent memory-exhaustion attacks
+    # limit defaults to DEFAULT_MAX_NUM, but developer can increase it via max_num
+    absolute_max = max(DEFAULT_MAX_NUM, max_num)
     attrs = {'form': form, 'extra': extra,
              'can_order': can_order, 'can_delete': can_delete,
-             'max_num': max_num}
+             'max_num': max_num, 'absolute_max': absolute_max}
     return type(form.__name__ + 'FormSet', (formset,), attrs)
 
 def all_valid(formsets):
