@@ -34,22 +34,3 @@ class ConditionalGetMiddleware(object):
                     response.status_code = 304
 
         return response
-
-class SetRemoteAddrFromForwardedFor(object):
-    """
-    This middleware has been removed; see the Django 1.1 release notes for
-    details.
-    
-    It previously set REMOTE_ADDR based on HTTP_X_FORWARDED_FOR. However, after
-    investiagtion, it turns out this is impossible to do in a general manner:
-    different proxies treat the X-Forwarded-For header differently. Thus, a
-    built-in middleware can lead to application-level security problems, and so
-    this was removed in Django 1.1
-    
-    """
-    def __init__(self):
-        import warnings
-        warnings.warn("SetRemoteAddrFromForwardedFor has been removed. "
-                      "See the Django 1.1 release notes for details.",
-                      category=DeprecationWarning)
-        raise MiddlewareNotUsed()
\ No newline at end of file
