@@ -1,20 +1,22 @@
 #! -*- coding: utf-8 -*-
-import os
 import errno
+import base64
+import os
 import shutil
-import unittest
 from StringIO import StringIO
 
 from django.core.files import temp as tempfile
 from django.core.files.uploadedfile import SimpleUploadedFile
+from django.http.multipartparser import MultiPartParser
 from django.test import TestCase, client
 from django.utils import simplejson
+from django.utils import unittest
 from django.utils.hashcompat import sha_constructor
-from django.http.multipartparser import MultiPartParser
 
 from models import FileModel, temp_storage, UPLOAD_TO
 import uploadhandler
 
+
 UNICODE_FILENAME = u'test-0123456789_中文_Orléans.jpg'
 
 class FileUploadTests(TestCase):
@@ -54,6 +56,30 @@ class FileUploadTests(TestCase):
 
         self.assertEqual(response.status_code, 200)
 
+    def test_base64_upload(self):
+        test_string = "This data will be transmitted base64-encoded."
+        payload = "\r\n".join([
+            '--' + client.BOUNDARY,
+            'Content-Disposition: form-data; name="file"; filename="test.txt"',
+            'Content-Type: application/octet-stream',
+            'Content-Transfer-Encoding: base64',
+            '',
+            base64.b64encode(test_string),
+            '--' + client.BOUNDARY + '--',
+            '',
+        ])
+        r = {
+            'CONTENT_LENGTH': len(payload),
+            'CONTENT_TYPE':   client.MULTIPART_CONTENT,
+            'PATH_INFO':      "/file_uploads/echo_content/",
+            'REQUEST_METHOD': 'POST',
+            'wsgi.input':     client.FakePayload(payload),
+        }
+        response = self.client.request(**r)
+        received = simplejson.loads(response.content)
+
+        self.assertEqual(received['file'], test_string)
+
     def test_unicode_file_name(self):
         tdir = tempfile.gettempdir()
 
@@ -147,7 +173,7 @@ class FileUploadTests(TestCase):
             'wsgi.input':     client.FakePayload(payload),
         }
         got = simplejson.loads(self.client.request(**r).content)
-        self.assert_(len(got['file']) < 256, "Got a long file name (%s characters)." % len(got['file']))
+        self.assertTrue(len(got['file']) < 256, "Got a long file name (%s characters)." % len(got['file']))
 
     def test_custom_upload_handler(self):
         # A small file (under the 5M quota)
@@ -163,12 +189,12 @@ class FileUploadTests(TestCase):
         # Small file posting should work.
         response = self.client.post('/file_uploads/quota/', {'f': smallfile})
         got = simplejson.loads(response.content)
-        self.assert_('f' in got)
+        self.assertTrue('f' in got)
 
         # Large files don't go through.
         response = self.client.post("/file_uploads/quota/", {'f': bigfile})
         got = simplejson.loads(response.content)
-        self.assert_('f' not in got)
+        self.assertTrue('f' not in got)
 
     def test_broken_custom_upload_handler(self):
         f = tempfile.NamedTemporaryFile()
@@ -223,7 +249,7 @@ class FileUploadTests(TestCase):
                 ret = super(POSTAccessingHandler, self).handle_uncaught_exception(request, resolver, exc_info)
                 p = request.POST
                 return ret
-        
+
         post_data = {
             'name': 'Ringo',
             'file_field': open(__file__),
@@ -244,20 +270,13 @@ class FileUploadTests(TestCase):
             response = self.client.post('/file_uploads/upload_errors/', post_data)
         except reference_error.__class__, err:
             self.failIf(
-                str(err) == str(reference_error), 
+                str(err) == str(reference_error),
                 "Caught a repeated exception that'll cause an infinite loop in file uploads."
             )
         except Exception, err:
             # CustomUploadError is the error that should have been raised
             self.assertEqual(err.__class__, uploadhandler.CustomUploadError)
 
-    def setUp(self):
-        if not os.path.isdir(temp_storage.location):
-            os.makedirs(temp_storage.location)
-
-    def tearDown(self):
-        shutil.rmtree(temp_storage.location)
-
 class DirectoryCreationTests(unittest.TestCase):
     """
     Tests for error handling during directory creation
@@ -267,6 +286,9 @@ class DirectoryCreationTests(unittest.TestCase):
         self.obj = FileModel()
         if not os.path.isdir(temp_storage.location):
             os.makedirs(temp_storage.location)
+        if os.path.isdir(UPLOAD_TO):
+            os.chmod(UPLOAD_TO, 0700)
+            shutil.rmtree(UPLOAD_TO)
 
     def tearDown(self):
         os.chmod(temp_storage.location, 0700)
@@ -278,7 +300,7 @@ class DirectoryCreationTests(unittest.TestCase):
         try:
             self.obj.testfile.save('foo.txt', SimpleUploadedFile('foo.txt', 'x'))
         except OSError, err:
-            self.assertEquals(err.errno, errno.EACCES)
+            self.assertEqual(err.errno, errno.EACCES)
         except Exception, err:
             self.fail("OSError [Errno %s] not raised." % errno.EACCES)
 
@@ -292,7 +314,7 @@ class DirectoryCreationTests(unittest.TestCase):
         except IOError, err:
             # The test needs to be done on a specific string as IOError
             # is raised even without the patch (just not early enough)
-            self.assertEquals(err.args[0],
+            self.assertEqual(err.args[0],
                               "%s exists and is not a directory." % UPLOAD_TO)
         except:
             self.fail("IOError not raised")
