@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
-from unittest import TestCase
-from django.forms import Form, CharField, IntegerField, ValidationError, DateField
+from django.forms import Form, CharField, IntegerField, ValidationError, DateField, formsets
 from django.forms.formsets import formset_factory, BaseFormSet
+from django.utils.unittest import TestCase
 
 
 class Choice(Form):
@@ -41,20 +41,13 @@ FavoriteDrinksFormSet = formset_factory(FavoriteDrinkForm,
     formset=BaseFavoriteDrinksFormSet, extra=3)
 
 
-class ArticleForm(Form):
-    title = CharField()
-    pub_date = DateField()
-
-ArticleFormSet = formset_factory(ArticleForm)
-
-
 class FormsFormsetTestCase(TestCase):
     def test_basic_formset(self):
         # A FormSet constructor takes the same arguments as Form. Let's create a FormSet
         # for adding data. By default, it displays 1 blank form. It can display more,
         # but we'll look at how to do so later.
         formset = ChoiceFormSet(auto_id=False, prefix='choices')
-        self.assertEqual(str(formset), """<input type="hidden" name="choices-TOTAL_FORMS" value="1" /><input type="hidden" name="choices-INITIAL_FORMS" value="0" /><input type="hidden" name="choices-MAX_NUM_FORMS" />
+        self.assertEqual(str(formset), """<input type="hidden" name="choices-TOTAL_FORMS" value="1" /><input type="hidden" name="choices-INITIAL_FORMS" value="0" /><input type="hidden" name="choices-MAX_NUM_FORMS" value="1000" />
 <tr><th>Choice:</th><td><input type="text" name="choices-0-choice" /></td></tr>
 <tr><th>Votes:</th><td><input type="text" name="choices-0-votes" /></td></tr>""")
 
@@ -630,8 +623,8 @@ class FormsFormsetTestCase(TestCase):
         # Limiting the maximum number of forms ########################################
         # Base case for max_num.
 
-        # When not passed, max_num will take its default value of None, i.e. unlimited
-        # number of forms, only controlled by the value of the extra parameter.
+        # When not passed, max_num will take a high default value, leaving the
+        # number of forms only controlled by the value of the extra parameter.
 
         LimitedFavoriteDrinkFormSet = formset_factory(FavoriteDrinkForm, extra=3)
         formset = LimitedFavoriteDrinkFormSet()
@@ -678,8 +671,8 @@ class FormsFormsetTestCase(TestCase):
     def test_max_num_with_initial_data(self):
         # max_num with initial data
 
-        # When not passed, max_num will take its default value of None, i.e. unlimited
-        # number of forms, only controlled by the values of the initial and extra
+        # When not passed, max_num will take a high default value, leaving the
+        # number of forms only controlled by the value of the initial and extra
         # parameters.
 
         initial = [
@@ -753,7 +746,12 @@ class FormsFormsetTestCase(TestCase):
         formset = FavoriteDrinksFormSet()
         self.assertEqual(formset.management_form.prefix, 'form')
 
-        formset = FavoriteDrinksFormSet(data={})
+        data = {
+            'form-TOTAL_FORMS': '2',
+            'form-INITIAL_FORMS': '0',
+            'form-MAX_NUM_FORMS': '0',
+        }
+        formset = FavoriteDrinksFormSet(data=data)
         self.assertEqual(formset.management_form.prefix, 'form')
 
         formset = FavoriteDrinksFormSet(initial={})
@@ -774,27 +772,170 @@ class FormsFormsetTestCase(TestCase):
         self.assertFalse(formset.is_valid())
         self.assertEqual(formset.non_form_errors(), [u'You may only specify a drink once.'])
 
-    def test_empty_forms_are_unbound(self):
+    def test_formset_iteration(self):
+        # Regression tests for #16455 -- formset instances are iterable
+        ChoiceFormset = formset_factory(Choice, extra=3)
+        formset = ChoiceFormset()
+
+        # confirm iterated formset yields formset.forms
+        forms = list(formset)
+        self.assertEqual(forms, formset.forms)
+        self.assertEqual(len(formset), len(forms))
+
+        # confirm indexing of formset
+        self.assertEqual(formset[0], forms[0])
+        try:
+            formset[3]
+            self.fail('Requesting an invalid formset index should raise an exception')
+        except IndexError:
+            pass
+
+        # Formets can override the default iteration order
+        class BaseReverseFormSet(BaseFormSet):
+            def __iter__(self):
+                for form in reversed(self.forms):
+                    yield form
+
+        ReverseChoiceFormset = formset_factory(Choice, BaseReverseFormSet, extra=3)
+        reverse_formset = ReverseChoiceFormset()
+
+        # confirm that __iter__ modifies rendering order
+        # compare forms from "reverse" formset with forms from original formset
+        self.assertEqual(str(reverse_formset[0]), str(forms[-1]))
+        self.assertEqual(str(reverse_formset[1]), str(forms[-2]))
+        self.assertEqual(len(reverse_formset), len(forms))
+
+    def test_hard_limit_on_instantiated_forms(self):
+        """A formset has a hard limit on the number of forms instantiated."""
+        # reduce the default limit of 1000 temporarily for testing
+        _old_DEFAULT_MAX_NUM = formsets.DEFAULT_MAX_NUM
+        try:
+            formsets.DEFAULT_MAX_NUM = 3
+            ChoiceFormSet = formset_factory(Choice)
+            # someone fiddles with the mgmt form data...
+            formset = ChoiceFormSet(
+                {
+                    'choices-TOTAL_FORMS': '4',
+                    'choices-INITIAL_FORMS': '0',
+                    'choices-MAX_NUM_FORMS': '4',
+                    'choices-0-choice': 'Zero',
+                    'choices-0-votes': '0',
+                    'choices-1-choice': 'One',
+                    'choices-1-votes': '1',
+                    'choices-2-choice': 'Two',
+                    'choices-2-votes': '2',
+                    'choices-3-choice': 'Three',
+                    'choices-3-votes': '3',
+                    },
+                prefix='choices',
+                )
+            # But we still only instantiate 3 forms
+            self.assertEqual(len(formset.forms), 3)
+        finally:
+            formsets.DEFAULT_MAX_NUM = _old_DEFAULT_MAX_NUM
+
+    def test_increase_hard_limit(self):
+        """Can increase the built-in forms limit via a higher max_num."""
+        # reduce the default limit of 1000 temporarily for testing
+        _old_DEFAULT_MAX_NUM = formsets.DEFAULT_MAX_NUM
+        try:
+            formsets.DEFAULT_MAX_NUM = 3
+            # for this form, we want a limit of 4
+            ChoiceFormSet = formset_factory(Choice, max_num=4)
+            formset = ChoiceFormSet(
+                {
+                    'choices-TOTAL_FORMS': '4',
+                    'choices-INITIAL_FORMS': '0',
+                    'choices-MAX_NUM_FORMS': '4',
+                    'choices-0-choice': 'Zero',
+                    'choices-0-votes': '0',
+                    'choices-1-choice': 'One',
+                    'choices-1-votes': '1',
+                    'choices-2-choice': 'Two',
+                    'choices-2-votes': '2',
+                    'choices-3-choice': 'Three',
+                    'choices-3-votes': '3',
+                    },
+                prefix='choices',
+                )
+            # This time four forms are instantiated
+            self.assertEqual(len(formset.forms), 4)
+        finally:
+            formsets.DEFAULT_MAX_NUM = _old_DEFAULT_MAX_NUM
+
+
+data = {
+    'choices-TOTAL_FORMS': '1', # the number of forms rendered
+    'choices-INITIAL_FORMS': '0', # the number of forms with initial data
+    'choices-MAX_NUM_FORMS': '0', # max number of forms
+    'choices-0-choice': 'Calexico',
+    'choices-0-votes': '100',
+}
+
+class Choice(Form):
+    choice = CharField()
+    votes = IntegerField()
+
+ChoiceFormSet = formset_factory(Choice)
+
+class FormsetAsFooTests(TestCase):
+    def test_as_table(self):
+        formset = ChoiceFormSet(data, auto_id=False, prefix='choices')
+        self.assertEqual(formset.as_table(),"""<input type="hidden" name="choices-TOTAL_FORMS" value="1" /><input type="hidden" name="choices-INITIAL_FORMS" value="0" /><input type="hidden" name="choices-MAX_NUM_FORMS" value="0" />
+<tr><th>Choice:</th><td><input type="text" name="choices-0-choice" value="Calexico" /></td></tr>
+<tr><th>Votes:</th><td><input type="text" name="choices-0-votes" value="100" /></td></tr>""")
+
+    def test_as_p(self):
+        formset = ChoiceFormSet(data, auto_id=False, prefix='choices')
+        self.assertEqual(formset.as_p(),"""<input type="hidden" name="choices-TOTAL_FORMS" value="1" /><input type="hidden" name="choices-INITIAL_FORMS" value="0" /><input type="hidden" name="choices-MAX_NUM_FORMS" value="0" />
+<p>Choice: <input type="text" name="choices-0-choice" value="Calexico" /></p>
+<p>Votes: <input type="text" name="choices-0-votes" value="100" /></p>""")
+
+    def test_as_ul(self):
+        formset = ChoiceFormSet(data, auto_id=False, prefix='choices')
+        self.assertEqual(formset.as_ul(),"""<input type="hidden" name="choices-TOTAL_FORMS" value="1" /><input type="hidden" name="choices-INITIAL_FORMS" value="0" /><input type="hidden" name="choices-MAX_NUM_FORMS" value="0" />
+<li>Choice: <input type="text" name="choices-0-choice" value="Calexico" /></li>
+<li>Votes: <input type="text" name="choices-0-votes" value="100" /></li>""")
+
+
+# Regression test for #11418 #################################################
+class ArticleForm(Form):
+    title = CharField()
+    pub_date = DateField()
+
+ArticleFormSet = formset_factory(ArticleForm)
+
+class TestIsBoundBehavior(TestCase):
+    def test_no_data_raises_validation_error(self):
+        self.assertRaises(ValidationError, ArticleFormSet, {})
+
+    def test_with_management_data_attrs_work_fine(self):
         data = {
             'form-TOTAL_FORMS': u'1',
             'form-INITIAL_FORMS': u'0',
+        }
+        formset = ArticleFormSet(data)
+        self.assertEqual(0, formset.initial_form_count())
+        self.assertEqual(1, formset.total_form_count())
+        self.assertTrue(formset.is_bound)
+        self.assertTrue(formset.forms[0].is_bound)
+        self.assertTrue(formset.is_valid())
+        self.assertTrue(formset.forms[0].is_valid())
+        self.assertEqual([{}], formset.cleaned_data)
+
+
+    def test_form_errors_are_cought_by_formset(self):
+        data = {
+            'form-TOTAL_FORMS': u'2',
+            'form-INITIAL_FORMS': u'0',
             'form-0-title': u'Test',
             'form-0-pub_date': u'1904-06-16',
+            'form-1-title': u'Test',
+            'form-1-pub_date': u'', # <-- this date is missing but required
         }
-        unbound_formset = ArticleFormSet()
-        bound_formset = ArticleFormSet(data)
-
-        empty_forms = []
-
-        empty_forms.append(unbound_formset.empty_form)
-        empty_forms.append(bound_formset.empty_form)
-
-        # Empty forms should be unbound
-        self.assertFalse(empty_forms[0].is_bound)
-        self.assertFalse(empty_forms[1].is_bound)
-
-        # The empty forms should be equal.
-        self.assertEqual(empty_forms[0].as_p(), empty_forms[1].as_p())
+        formset = ArticleFormSet(data)
+        self.assertFalse(formset.is_valid())
+        self.assertEqual([{}, {'pub_date': [u'This field is required.']}], formset.errors)
 
     def test_empty_forms_are_unbound(self):
         data = {
@@ -818,12 +959,12 @@ class FormsFormsetTestCase(TestCase):
         # The empty forms should be equal.
         self.assertEqual(empty_forms[0].as_p(), empty_forms[1].as_p())
 
-class TestEmptyFormSet(TestCase): 
+class TestEmptyFormSet(TestCase):
     "Test that an empty formset still calls clean()"
-    def test_empty_formset_is_valid(self): 
-        EmptyFsetWontValidateFormset = formset_factory(FavoriteDrinkForm, extra=0, formset=EmptyFsetWontValidate) 
-        formset = EmptyFsetWontValidateFormset(data={'form-INITIAL_FORMS':'0', 'form-TOTAL_FORMS':'0'},prefix="form") 
-        formset2 = EmptyFsetWontValidateFormset(data={'form-INITIAL_FORMS':'0', 'form-TOTAL_FORMS':'1', 'form-0-name':'bah' },prefix="form") 
-        self.assertFalse(formset.is_valid()) 
+    def test_empty_formset_is_valid(self):
+        EmptyFsetWontValidateFormset = formset_factory(FavoriteDrinkForm, extra=0, formset=EmptyFsetWontValidate)
+        formset = EmptyFsetWontValidateFormset(data={'form-INITIAL_FORMS':'0', 'form-TOTAL_FORMS':'0'},prefix="form")
+        formset2 = EmptyFsetWontValidateFormset(data={'form-INITIAL_FORMS':'0', 'form-TOTAL_FORMS':'1', 'form-0-name':'bah' },prefix="form")
+        self.assertFalse(formset.is_valid())
         self.assertFalse(formset2.is_valid())
 
